
/*
 *  Project     [tie]
 *  Author      TriX
 *  Description TI92 linker for the parallel & serial cable
 *  Module      lowlevel driver for ti92 link interface
 */

#ifndef DJGPP
#error  Currently only for DJGPP availabel
#endif

#include "ti92.h"
#include <dos.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>

int SetPort (int Port)
{
   char            DeviceName[32];

   if (Port & COM_PORT_1)
   {
      sprintf (DeviceName, "/dev/ttyS%d", Port - COM_PORT_1);
      if ((Device = fopen (DeviceName, "r+")) == NULL)
      {
         printf ("Error opening communication port\n");
         exit (1);
      } /* if */
   } else {
      /* LPT Port */
      Device = NULL;
      dosmemget (0x408 + (Port - LPT_PORT_1) * 2, 2, &LPT_Port);
   } /* if */
   return 0;
} /* SetPort */

int ClosePort ()
{
   if (Device != NULL) fclose (Device);
   return 0;
} /* ClosePort */

#define  lptin  LPT_Port + 1
#define  lptout LPT_Port
/* Wait for the TI92 Acknowledge (LPT Routine) */
int Wait (int Code, int Until)
{
   struct time t;
   long   timeout = 1 << 30;

   while ((inportb (lptin) & Code) != Until && --timeout > 0);
   if (timeout == 0)
      return 0;
   else
      return 1;
} /* Wait */


int SendByte (unsigned char byte)
{
   if (Device != NULL)
      fputc (byte, Device);
   else {
      /* LPT putbyte routine */
     int n, b = byte;
     for (n = 0; n < 8; n++)
     {
        outportb (lptout, 0x3);
        if ((b & (1 << n)) == 0)
        {
           /* Bit not set */
           outportb (lptout, 0x1);
           /* Acknowledge from the TI92 */
           if (!Wait (0x20, 0)) return 0;
           outportb (lptout, 0x3);
           if (!Wait (0x20, 0x20)) return 0;
        } else {
           /* Bit set */
           outportb (lptout, 0x2);
           /* Acknowledge from the TI92 */
           if (!Wait (0x10, 0)) return 0;
           outportb (lptout, 0x3);
           if (!Wait (0x10, 0x10)) return 0;
        } /* if */
     } /* for */
   } /* if */
   return 1;
} /* SendByte */

int GetByte (unsigned char *byte)
{
   if (Device != NULL)
      *byte = fgetc (Device);
   else {
      /* LPT putbyte routine */
      int n, x = 0, i;
      long timeout;

      for (n = 0; n < 8; n++)
      {
         i = 0x30;
         timeout = 1 << 30;
         while ((i & 0x30) == 0x30 && timeout > 0)
         {
            i = inportb (lptin) & 0x30;
            timeout--;
         } /* while */
         if (i == 0x30 || timeout == 0) return 0;
         if (i == 0x10)
         {
            /* Bit = 1 */
            x += (1 << n);
            outportb (lptout, 0x1);
            if (!Wait (0x20, 0x20)) return 0;
         } else {
            /* Bit = 0 */
            outportb (lptout, 0x2);
            if (!Wait (0x10, 0x10)) return 0;
         } /* if */
         outportb (lptout, 0x3);
      } /* for */
      *byte = (unsigned char) x;
   } /* if */
   return 1;
} /* GetByte */
